/* java_ser1d.c */

#include <stdio.h>
#include <stdlib.h>
#include "sockio.h"
#include "binio.h"

/*********************************************************
 * java_ser: connect to server and send the data
 *     
 *
 * return value (all functions)
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 *********************************************************/



/*************************************************************
 * java_fser_
 * c program to link with fortran 90
 * This program writes the x and fx values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 ************************************************************/
/* ONLY SUPPORT BIG ENDIAN */
int java_fser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   size = *nx;
   
   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&fx[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser_
 * c program to link with fortran 90
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_ser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 10;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)fx[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_mser_(name, nt, time, x, fx, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double fx[];
  int nx[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, i, j, index;
   int command = 12;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)fx[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_cmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   fx   - fx value array
 *   cx   - color value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_cmser_(name, nt, time, x, fx, cx, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double fx[];
  double cx[];
  int nx[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, i, j, index;
   int command = 20;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)fx[index]);
         write_float4(sockfd, (float)cx[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbser_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   bb    - x min and max  value
 *   xmax    - x max value
 *   nx      - number of x points 
 *   fx     - fxy value array
 ************************************************************/
int java_bbser_(name, time, bb, nx, fx) 
  char *name;
  double *time;
  double bb[];
  int *nx;
  double fx[];
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 28;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_float4(sockfd, (float)bb[0]);
   write_float4(sockfd, (float)bb[1]);



   /* write fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)fx[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   bb    - x min and max array
 *   nx    -   nx value array
 *   fx   - fxy value array
 ************************************************************/
int java_bbmser_(name, nt, time, bb, nx, fx) 
  char *name;
  int *nt;
  double time[];
  double bb[][4];
  int nx[];
  double fx[];

{
   int sockfd;
   int ntsize = *nt, name_length, i, j, index;
   int command = 29;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);


   index = 0;
   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      write_float4(sockfd, (float)bb[i][0]);
      write_float4(sockfd, (float)bb[i][1]);
     
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)fx[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_pser_
 * c program to link with fortran 90
 * to be used for parallel machine
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *   p    - total number of processors
 ************************************************************/
int java_pser_(name, time, x, fx, nx, p) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
  int *p;
{
   int sockfd;
   int size = *nx, name_length, i, pp = *p;
   double ft = *time;
   float ftime = (float)ft;
   int command = 17;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time, nx, and p */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_int4(sockfd, pp);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)fx[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_vser_
 * c program to link with fortran 90
 * This program writes the vector data to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - r value array
 *   nx   - size of x, y, r, t array
 ************************************************************/
int java_vser_(name, time, x, y, r, t, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double r[];
  double t[];
  int *nx;
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 22;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)r[i]);
      write_float4(sockfd, (float)t[i]);
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_vmser_
 * c program to link with fortran 90
 * sends multiple vector data to the server
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - t value array
 *   nx   - size of x, y, r, t array
 ************************************************************/
int java_vmser_(name, nt, time, x, y, r, t, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double r[];
  double t[];
  int nx[];
{
   int sockfd;
   int ntsize = *nt, name_length, i, j, index;
   int command = 27;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)y[index]);
         write_float4(sockfd, (float)r[index]);
         write_float4(sockfd, (float)t[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_sercon_
 * c program to link with fortran 90
 * This program writes the connection list to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   c    - color value array
 *   nx   - size of x, y, c array
 *   cl   - connection list
 *   nc   - the number of elements of connection list
 ************************************************************/
int java_sercon_(name, time, x, y, c, nx, cl, nc) 
  char *name;
  double *time;
  double x[];
  double y[];
  double c[];
  int *nx;
  int cl[];
  int *nc;
{
   int sockfd;
   int size = *nx, con = *nc, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 19;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx, nc */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_int4(sockfd, con);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)c[i]);
   }

   for (i=0; i<con; i++) 
      write_int4(sockfd, cl[i]);

   
   close(sockfd);
   return 0;
}





/*************************************************************
 * java_eser_
 * c program to link with fortran 90
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   ex   - error value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_eser_(name, time, x, fx, ex, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  double ex[];
  int *nx;
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 35;

   sockfd = initialize();
   if (sockfd == -1)
     return 1;

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)fx[i]);
      write_float4(sockfd, (float)ex[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_emser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   fx   - fx value array
 *   ex   - error value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_emser_(name, nt, time, x, fx, ex, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double fx[];
  double ex[];
  int nx[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, i, j, index;
   int command = 36;

   sockfd = initialize();
   if (sockfd == -1)
     return 1;

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)fx[index]);
         write_float4(sockfd, (float)ex[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}



/*************************************************************
 * java_ebbser_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   bb    - x min and max  value
 *   xmax    - x max value
 *   nx      - number of x points 
 *   fx     - fx value array
 *   ex     - error value array
 ************************************************************/
int java_ebbser_(name, time, bb, nx, fx, ex) 
  char *name;
  double *time;
  double bb[];
  int *nx;
  double fx[];
  double ex[];
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 37;

   sockfd = initialize();
   if (sockfd == -1)
     return 1;

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_float4(sockfd, (float)bb[0]);
   write_float4(sockfd, (float)bb[1]);



   /* write fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)fx[i]);
      write_float4(sockfd, (float)ex[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_ebbmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   bb    - x min and max array
 *   nx    -   nx value array
 *   fx   - fx value array
 *   ex   - error value array
 ************************************************************/
int java_ebbmser_(name, nt, time, bb, nx, fx, ex) 
  char *name;
  int *nt;
  double time[];
  double bb[][2];
  int nx[];
  double fx[];
  double ex[];
{
   int sockfd;
   int ntsize = *nt, name_length, i, j, index;
   int command = 38;

   sockfd = initialize();
   if (sockfd == -1)
     return 1;

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   index = 0;
   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      write_float4(sockfd, (float)bb[i][0]);
      write_float4(sockfd, (float)bb[i][1]);
     
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)fx[index]);
         write_float4(sockfd, (float)ex[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}



int java_eser(char *name, double time, double x[], double y[], double e[], int n) {
   return java_eser_(name, &time, x, y, e, &n);
}

int java_emser(char *name, int nt, double time[], double x[], double fx[], double ex[], int nx[]) {
   return java_emser_(name, &nt, time, x, fx, ex, nx);
}

int java_ebbser(char *name, double time, double bb[], int nx, double fx[], double ex[]) {
  java_ebbser_(name, &time, bb, &nx, fx, ex);
}

int java_ebbmser(char *name, int nt, double time[], double bb[][2], int nx[], double fx[], double ex[]) {
   java_ebbmser_(name, &nt, time, bb, nx, fx, ex);
}


int java_ser(char *name, double time, double x[], double y[], int n) {
   return java_ser_(name, &time, x, y, &n);
}

int java_mser(char *name, int nt, double time[], double x[], double fx[], int nx[]) {
   return java_mser_(name, &nt, time, x, fx, nx);
}

int java_cmser(char *name, int nt, double time[], double x[], double fx[], double cx[], int nx[]) {
   return java_cmser_(name, &nt, time, x, fx, cx, nx);
}


int java_bbser(char *name, double time, double bb[], int nx, double fx[]) {
  java_bbser_(name, &time, bb, &nx, fx);
}

int java_bbmser(char *name, int nt, double time[], double bb[][4], int nx[], double fx[]) {
   java_bbmser_(name, &nt, time, bb, nx, fx);
}

int java_pser(char *name, double time, double x[], double fx[], int nx, int p) {
   java_pser_(name, &time, x, fx, &nx, &p);
}


int java_vser(char *name, double time, double x[], double y[], double r[], double t[], int nx) {
   java_vser_(name, &time, x, y, r, t, &nx);
}

java_vmser(char *name, int nt, double time[], double x[], double y[], double r[], double t[], int nx[]) {
   java_vmser_(name, &nt, time, x, y, r, t, nx);
}


int java_sercon(char *name, double time, double x[], double y[], double c[], int nx, int cl[], int nc) {
   java_sercon_(name, &time, x, y, c, &nx, cl, &nc);
}
/* java_ser2d.c */

#include <stdio.h>
#include <stdlib.h>
#include "sockio.h"
#include "binio.h"

/*********************************************************
 * java_ser2d: connect to server and send the data
 *     
 *
 * return value (all functions)
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 *********************************************************/


/*************************************************************
 * java_fser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x, y and fx array
 ************************************************************/
/* ONLY SUPPORT BIG ENDIAN */
int java_fser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }
   size = *nx;

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&y[i], sizeof(double), 1, fp);
      fwrite(&fxy[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x and fx array
 ************************************************************/

int java_ser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 15;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   fxy   - fxy value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_mser2d_(name, nt, time, x, y, fxy, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double fxy[];
  int nx[];
{

   int sockfd;
   int ntsize = *nt;
   int name_length, i, j, index;
   int command = 16;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)y[index]);
         write_float4(sockfd, (float)fxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 * (reduced form of java_ser2d)
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   nx   - x array size
 *   y    - y value array
 *   ny   - y array size
 *   fxy  - fxy value array
 ************************************************************/
int java_bser2d_(name, time, x, nx, y, ny, fxy) 
  char *name;
  double *time;
  double x[];
  int *nx;
  double y[];
  int *ny;
  double fxy[];
{
   int sockfd;
   int size, xsize = *nx, ysize = *ny, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 30;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time */
   write_float4(sockfd, ftime);

   /* send x array */
   write_int4(sockfd, xsize);
   for (i=0; i<xsize; i++) 
      write_float4(sockfd, (float)x[i]);

   /* send y array */
   write_int4(sockfd, ysize);
   for (i=0; i<ysize; i++) 
      write_float4(sockfd, (float)y[i]);

   /* send fxy array */
   size = xsize * ysize;
   write_int4(sockfd, size);
   for (i=0; i<size; i++) 
      write_float4(sockfd, (float)fxy[i]);

   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bmser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   nx   - size of x array
 *   y    - y value array
 *   ny   - size of y array 
 *   fxy   - fxy value array
 ************************************************************/
int java_bmser2d_(name, nt, time, x, nx, y, ny, fxy) 
  char *name;
  int *nt;
  double time[];
  double x[];
  int nx[];
  double y[];
  int ny[];
  double fxy[];
{
  
   int sockfd;
   int ntsize = *nt, size, name_length, i, j, index, xindex, yindex;
   int command = 31;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;
   xindex = 0;
   yindex = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[xindex]);
         xindex++;
      }

      write_int4(sockfd, ny[i]);
      for (j=0; j<ny[i]; j++) {
         write_float4(sockfd, (float)y[yindex]);
         yindex++;
      }

      size = nx[i] * ny[i];
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)fxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}

/*************************************************************
 * java_bbser2d_
 * c program to link with fortran 90
 * sends 2d bounding box and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   bb    - x min, max, y min, and y max
 *   xmax    - x max value
 *   nx      - number of x points 
 *   ymin    - y min value
 *   ymax    - y max value
 *   ny      - number of y points 
 *   fxy     - fxy value array
 ************************************************************/
int java_bbser2d_(name, time, bb, nx, ny, fxy) 
  char *name;
  double *time;
  double bb[];
  int *nx;
  int *ny;
  double fxy[];
{
   int sockfd;
   int size, xsize = *nx, ysize = *ny, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 25;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   size = xsize * ysize;
   /* write time, x and y size, and bb */
   write_float4(sockfd, ftime);
   write_int4(sockfd, xsize);
   write_int4(sockfd, ysize);
   write_float4(sockfd, (float)bb[0]);
   write_float4(sockfd, (float)bb[1]);
   write_float4(sockfd, (float)bb[2]);
   write_float4(sockfd, (float)bb[3]);

   /* write fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}

/*************************************************************
 * java_bbmser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   bb    - x min, max, y min, and y max value array
 *   nx    -   nx value array
 *   ny    -   ny value array
 *   fxy   - fxy value array
 ************************************************************/
int java_bbmser2d_(name, nt, time, bb, nx, ny, fxy) 
  char *name;
  int *nt;
  double time[];
  double bb[][4];
  int nx[];
  int ny[];
  double fxy[];

{
   int sockfd;
   int ntsize = *nt, size, name_length, i, j, index;
   int command = 26;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   index = 0;
   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, time[i]);
      write_int4(sockfd, nx[i]);
      write_int4(sockfd, ny[i]);
      write_float4(sockfd, (float)bb[i][0]);
      write_float4(sockfd, (float)bb[i][1]);
      write_float4(sockfd, (float)bb[i][2]);
      write_float4(sockfd, (float)bb[i][3]);
     
      size = nx[i] * ny[i];
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)fxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_cbbmser2d_
 * sends data like java_bbmser2d.
 * It also sends the color data to the server. Each point can have 
 * a custom color with the specified RGB value, where the red component 
 * is in bits 16-23 of the argument, the green component is in bits 8-15 
 * of the argument, and the blue component is in bits 0-7. The value zero
 * indicates no contribution from the primary color component. 
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   bb    - x min, max, y min, and y max value array
 *   nx    -   nx value array
 *   ny    -   ny value array
 *   fxy   - fxy value array
 *   cxy   - color value array
 ************************************************************/
int java_cbbmser2d_(name, nt, time, bb, nx, ny, fxy, cxy) 
  char *name;
  int *nt;
  double time[];
  double bb[][4];
  int nx[];
  int ny[];
  double fxy[];
  double cxy[];

{
   int sockfd;
   int ntsize = *nt, size, name_length, i, j, index;
   int command = 33;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   index = 0;
   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, time[i]);
      write_int4(sockfd, nx[i]);
      write_int4(sockfd, ny[i]);
      write_float4(sockfd, (float)bb[i][0]);
      write_float4(sockfd, (float)bb[i][1]);
      write_float4(sockfd, (float)bb[i][2]);
      write_float4(sockfd, (float)bb[i][3]);
     
      size = nx[i] * ny[i];
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)fxy[index]);
         write_float4(sockfd, (float)cxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_pser_
 * c program to link with fortran 90
 * to be used for parallel machine
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *   p    - total number of processors
 ************************************************************/
int java_pser2d_(name, time, x, y, fxy, nx, p) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
  int *p;
{
   int sockfd;
   int size = *nx, name_length, i, pp = *p;
   double ft = *time;
   float ftime = (float)ft;
   int command = 18;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time, nx, and p */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_int4(sockfd, pp);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}



/*************************************************************
 * java_pbbser2d_
 * sends 2d bounding box and fxy values to socket.
 * used by parallel machine
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   bb    - x min, max, y min, and y max
 *   xmax    - x max value
 *   nx      - number of x points 
 *   ymin    - y min value
 *   ymax    - y max value
 *   ny      - number of y points 
 *   fxy     - fxy value array
 *   p       - the number of total processors
 ************************************************************/
int java_pbbser2d_(name, time, bb, nx, ny, fxy, p) 
  char *name;
  double *time;
  double bb[];
  int *nx;
  int *ny;
  double fxy[];
  int *p;
{
   int sockfd;
   int size, xsize = *nx, ysize = *ny, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int pp = *p;
   int command = 32;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   size = xsize * ysize;
   /* write time, x and y size, and bb */
   write_float4(sockfd, ftime);
   write_int4(sockfd, xsize);
   write_int4(sockfd, ysize);
   write_float4(sockfd, (float)bb[0]);
   write_float4(sockfd, (float)bb[1]);
   write_float4(sockfd, (float)bb[2]);
   write_float4(sockfd, (float)bb[3]);
   write_int4(sockfd, pp);

   /* write fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}



int java_ser2d(char *name, double time, double x[], double y[], double fxy[], int n) {
   return java_ser2d_(name, &time, x, y, fxy, &n);
}

int java_mser2d(char *name, int nt, double time[], double x[], double y[], double fxy[], int nx[]) {
   return java_mser2d_(name, &nt, time, x, y, fxy, nx);
}

int java_bser2d(char *name, double time, double x[], int nx, double y[], int ny, double fxy[]) {
  java_bser2d_(name, &time, x, &nx, y, &ny, fxy);
}

int java_bmser2d(char *name, int nt, double time[], double x[], int nx[], double y[], int ny[], double fxy[]) {
  java_bmser2d_(name, &nt, time, x, nx, y, ny, fxy);
}

int java_bbser2d(char *name, double time, double bb[], int nx, int ny, double fxy[]) {
  java_bbser2d_(name, &time, bb, &nx, &ny, fxy);
}

int java_bbmser2d(char *name, int nt, double time[], double bb[][4], int nx[], int ny[], double fxy[]) {
   java_bbmser2d_(name, &nt, time, bb, nx, ny, fxy);
}

int java_cbbmser2d(char *name, int nt, double time[], double bb[][4], int nx[], int ny[], double fxy[], double cxy[]) {
   java_cbbmser2d_(name, &nt, time, bb, nx, ny, fxy, cxy);
}


int java_pser2d(char *name, double time, double x[], double y[], double fx[], int nx, int p) {
   java_pser2d_(name, &time, x, y, fx, &nx, &p);
}


int java_pbbser2d(char *name, double time, double bb[], int nx, int ny, double fxy[], int p) {
   java_pbbser2d_(name, &time, bb, &nx, &ny, fxy, &p);
}



/* java_ser3d.c */

#include <stdio.h>
#include <stdlib.h>
#include "sockio.h"
#include "binio.h"


/*************************************************************
 * java_ser3d_
 * c program to link with fortran 90
 * This program writes the xsize, ysize, zsize and z values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   nx    - x dimension 
 *   ny    - y dimension 
 *   nz    - z dimension 
 *   threshold - initial threshold
 *   z   - z array
 ************************************************************/

int java_ser3d_(name, time, nx, ny, nz, threshold, grid) 
  char *name;
  double *time;
  int *nx;
  int *ny;
  int *nz; 
  double *threshold;
  double grid[];
{
   int sockfd;
   int nnx = *nx, nny = *ny, nnz = *nz, name_length, i;
   int size = nnx * nny * nnz;
   double ft = *time, thre = *threshold;
   float thresh = (float)thre;
   float ftime = (float)ft;
   int command = 1;

   sockfd = initialize2();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, 1);     /* send no of time */
   write_float4(sockfd, thresh);
   write_int4(sockfd, size);

   write_float4(sockfd, ftime);
   write_int4(sockfd, nnx);
   write_int4(sockfd, nny);
   write_int4(sockfd, nnz);

   /* write x and fx value */
   for (i=0; i<size; i++) 
      write_float4(sockfd, (float)grid[i]);
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser3d_
 * c program to link with fortran 90
 * This program writes the xsize, ysize, zsize and z values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   nx    - x dimension 
 *   ny    - y dimension 
 *   nz    - z dimension 
 *   threshold - initial threshold
 *   z   - z array
 ************************************************************/

int java_mser3d_(name, nt, time, nx, ny, nz, threshold, grid) 
  char *name;
  int *nt;
  double time[];
  int nx[];
  int ny[];
  int nz[]; 
  double *threshold;
  double grid[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, index, i, j, k, size, tot;
   double thre = *threshold;
   float thresh = (float)thre;
   int command = 1;

   sockfd = initialize2();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, ntsize);     /* send no of time */
   write_float4(sockfd, thresh);

   tot = 0;
   for (i=0; i<ntsize; i++) {
       tot += (nx[i] * ny[i] * nz[i]);
   }

   write_int4(sockfd, tot); 

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);
      write_int4(sockfd, nx[i]);
      write_int4(sockfd, ny[i]);
      write_int4(sockfd, nz[i]);

      size = nx[i] * ny[i] * nz[i];

      /* grid value */
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)grid[index]);
         index++;
      }   
   }
   
   close(sockfd);
   return 0;
}



/*************************************************************
 * java_sercon3d_
 * c program to link with fortran 90
 * This program writes the 3d connection list to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   z    - z value array
 *   c    - color value array
 *   nx   - size of x, y, z, and c array
 *   cl   - connection list
 *   nc   - the number of elements of connection list
 ************************************************************/
int java_sercon3d_(name, time, x, y, z, c, nx, cl, nc) 
  char *name;
  double *time;
  double x[];
  double y[];
  double z[];
  double c[];
  int *nx;
  int cl[];
  int *nc;
{
   int sockfd;
   int size = *nx, con = *nc, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 23;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx, nc */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_int4(sockfd, con);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)z[i]);
      write_float4(sockfd, (float)c[i]);
   }

   for (i=0; i<con; i++) 
      write_int4(sockfd, cl[i]);
   
   close(sockfd);
   return 0;
}


int java_ser3d(char *name, double time, int nx, int ny, int nz, double threshold, double grid[]) {
   return java_ser3d_(name, &time, &nx, &ny, &nz, &threshold, grid);
}

int java_mser3d(char *name, int nt, double time[], int nx[], int ny[], int nz[], double threshold, double grid[]) {
   return java_mser3d_(name, &nt, time, nx, ny, nz, &threshold, grid);
}

int java_sercon3d(char *name, double time, double x[], double y[], double z[], double c[], int nx, int cl[], int nc) {
   java_sercon3d_(name, &time, x, y, z, c, &nx, cl, &nc);
}


